@@ -13,15 +13,17 @@ API_URL = 'https://api.telegram.org/bot{0}/{1}'
 
 from datetime import datetime
 
-from telebot import util, logger
+from telebot import util
+import logging
 
+logger = logging.getLogger('TeleBot')
 
 proxy = None
 session = None
 
 FILE_URL = None
 
-REQUEST_TIMEOUT = None
+REQUEST_TIMEOUT = 300
 MAX_RETRIES = 3
 
 REQUEST_LIMIT = 50
@@ -56,10 +58,29 @@ class SessionManager:
 
 session_manager = SessionManager()
 
-async def _process_request(token, url, method='get', params=None, files=None, request_timeout=None):
+async def _process_request(token, url, method='get', params=None, files=None, **kwargs):
+    # Let's resolve all timeout parameters.
+    # getUpdates parameter may contain 2 parameters: request_timeout & timeout.
+    # other methods may contain timeout parameter that should be applied to
+    # ClientTimeout only.
+    # timeout should be added to params for getUpdates. All other timeout's should be used
+    # for request timeout.
+    # here we got request_timeout, so this is getUpdates method.
+    if 'request_timeout' in kwargs:
+        request_timeout = kwargs.pop('request_timeout')
+
+    else:
+        # let's check for timeout in params
+        request_timeout = params.pop('timeout', None) if params else None
+        # we will apply default request_timeout if there is no timeout in params
+        # otherwise, we will use timeout parameter applied for payload.
+    
+    request_timeout = REQUEST_TIMEOUT if request_timeout is None else request_timeout
+    
+
+    # Preparing data by adding all parameters and files to FormData
     params = _prepare_data(params, files)
-    if request_timeout is None:
-        request_timeout = REQUEST_TIMEOUT
+
     timeout = aiohttp.ClientTimeout(total=request_timeout)
     got_result = False
     current_try=0
@@ -159,15 +180,13 @@ async def get_file_url(token, file_id):
 async def download_file(token, file_path):
     if FILE_URL is None:
         url =  "https://api.telegram.org/file/bot{0}/{1}".format(token, file_path)
-    else:
-        # noinspection PyUnresolvedReferences
-        url =  FILE_URL.format(token, file_path)
-    async with await session_manager.get_session() as session:
-        async with session.get(url, proxy=proxy) as response:
-            result = await response.read()
-            if response.status != 200:
-                raise ApiHTTPException('Download file', result)
-        
+    else: url =  FILE_URL.format(token, file_path)
+    session = await session_manager.get_session()
+    async with session.get(url, proxy=proxy) as response:
+        if response.status != 200:
+            raise ApiHTTPException('Download file', result)
+        result = await response.read()
+    
     return result
 
 
@@ -259,23 +278,8 @@ async def send_message(
         token, chat_id, text,
         disable_web_page_preview=None, reply_to_message_id=None, reply_markup=None,
         parse_mode=None, disable_notification=None, timeout=None,
-        entities=None, allow_sending_without_reply=None, protect_content=None):
-    """
-    Use this method to send text messages. On success, the sent Message is returned.
-    :param token:
-    :param chat_id:
-    :param text:
-    :param disable_web_page_preview:
-    :param reply_to_message_id:
-    :param reply_markup:
-    :param parse_mode:
-    :param disable_notification:
-    :param timeout:
-    :param entities:
-    :param allow_sending_without_reply:
-    :param protect_content:
-    :return:
-    """
+        entities=None, allow_sending_without_reply=None, protect_content=None,
+        message_thread_id=None):
     method_name = 'sendMessage'
     params = {'chat_id': str(chat_id), 'text': text}
     if disable_web_page_preview is not None:
@@ -296,6 +300,8 @@ async def send_message(
         params['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         params['protect_content'] = protect_content
+    if message_thread_id:
+        params['message_thread_id'] = message_thread_id
     
     return await _process_request(token, method_name, params=params)
 
@@ -373,7 +379,8 @@ async def get_chat_member(token, chat_id, user_id):
 
 async def forward_message(
         token, chat_id, from_chat_id, message_id,
-        disable_notification=None, timeout=None, protect_content=None):
+        disable_notification=None, timeout=None, protect_content=None,
+        message_thread_id=None):
     method_url = r'forwardMessage'
     payload = {'chat_id': chat_id, 'from_chat_id': from_chat_id, 'message_id': message_id}
     if disable_notification is not None:
@@ -382,12 +389,14 @@ async def forward_message(
         payload['timeout'] = timeout
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload)
 
 
 async def copy_message(token, chat_id, from_chat_id, message_id, caption=None, parse_mode=None, caption_entities=None,
                  disable_notification=None, reply_to_message_id=None, allow_sending_without_reply=None,
-                 reply_markup=None, timeout=None, protect_content=None):
+                 reply_markup=None, timeout=None, protect_content=None, message_thread_id=None):
     method_url = r'copyMessage'
     payload = {'chat_id': chat_id, 'from_chat_id': from_chat_id, 'message_id': message_id}
     if caption is not None:
@@ -408,13 +417,16 @@ async def copy_message(token, chat_id, from_chat_id, message_id, caption=None, p
         payload['timeout'] = timeout
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload)
 
 
 async def send_dice(
         token, chat_id,
         emoji=None, disable_notification=None, reply_to_message_id=None,
-        reply_markup=None, timeout=None, allow_sending_without_reply=None, protect_content=None):
+        reply_markup=None, timeout=None, allow_sending_without_reply=None, protect_content=None,
+        message_thread_id=None):
     method_url = r'sendDice'
     payload = {'chat_id': chat_id}
     if emoji:
@@ -431,6 +443,8 @@ async def send_dice(
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload)
 
 
@@ -438,7 +452,8 @@ async def send_photo(
         token, chat_id, photo,
         caption=None, reply_to_message_id=None, reply_markup=None,
         parse_mode=None, disable_notification=None, timeout=None,
-        caption_entities=None, allow_sending_without_reply=None, protect_content=None):
+        caption_entities=None, allow_sending_without_reply=None, protect_content=None,
+        message_thread_id=None):
     method_url = r'sendPhoto'
     payload = {'chat_id': chat_id}
     files = None
@@ -466,13 +481,15 @@ async def send_photo(
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_media_group(
         token, chat_id, media,
         disable_notification=None, reply_to_message_id=None,
-        timeout=None, allow_sending_without_reply=None, protect_content=None):
+        timeout=None, allow_sending_without_reply=None, protect_content=None, message_thread_id=None):
     method_url = r'sendMediaGroup'
     media_json, files = await convert_input_media_array(media)
     payload = {'chat_id': chat_id, 'media': media_json}
@@ -486,6 +503,8 @@ async def send_media_group(
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(
         token, method_url, params=payload,
         method='post' if files else 'get',
@@ -497,7 +516,7 @@ async def send_location(
         live_period=None, reply_to_message_id=None, 
         reply_markup=None, disable_notification=None, 
         timeout=None, horizontal_accuracy=None, heading=None,
-        proximity_alert_radius=None, allow_sending_without_reply=None, protect_content=None):
+        proximity_alert_radius=None, allow_sending_without_reply=None, protect_content=None, message_thread_id=None):
     method_url = r'sendLocation'
     payload = {'chat_id': chat_id, 'latitude': latitude, 'longitude': longitude}
     if live_period:
@@ -520,6 +539,8 @@ async def send_location(
         payload['timeout'] = timeout
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload)
 
 
@@ -571,7 +592,7 @@ async def send_venue(
         foursquare_id=None, foursquare_type=None, disable_notification=None,
         reply_to_message_id=None, reply_markup=None, timeout=None,
         allow_sending_without_reply=None, google_place_id=None,
-        google_place_type=None, protect_content=None):
+        google_place_type=None, protect_content=None, message_thread_id=None):
     method_url = r'sendVenue'
     payload = {'chat_id': chat_id, 'latitude': latitude, 'longitude': longitude, 'title': title, 'address': address}
     if foursquare_id:
@@ -594,13 +615,15 @@ async def send_venue(
         payload['google_place_type'] = google_place_type
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload)
 
 
 async def send_contact(
         token, chat_id, phone_number, first_name, last_name=None, vcard=None,
         disable_notification=None, reply_to_message_id=None, reply_markup=None, timeout=None,
-        allow_sending_without_reply=None, protect_content=None):
+        allow_sending_without_reply=None, protect_content=None, message_thread_id=None):
     method_url = r'sendContact'
     payload = {'chat_id': chat_id, 'phone_number': phone_number, 'first_name': first_name}
     if last_name:
@@ -619,6 +642,8 @@ async def send_contact(
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload)
 
 
@@ -633,7 +658,7 @@ async def send_chat_action(token, chat_id, action, timeout=None):
 async def send_video(token, chat_id, data, duration=None, caption=None, reply_to_message_id=None, reply_markup=None,
                parse_mode=None, supports_streaming=None, disable_notification=None, timeout=None, 
                thumb=None, width=None, height=None, caption_entities=None, allow_sending_without_reply=None,
-               protect_content=None):
+               protect_content=None, message_thread_id=None):
     method_url = r'sendVideo'
     payload = {'chat_id': chat_id}
     files = None
@@ -675,13 +700,15 @@ async def send_video(token, chat_id, data, duration=None, caption=None, reply_to
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_animation(
         token, chat_id, data, duration=None, caption=None, reply_to_message_id=None, reply_markup=None,
         parse_mode=None, disable_notification=None, timeout=None, thumb=None, caption_entities=None,
-        allow_sending_without_reply=None, width=None, height=None, protect_content=None):
+        allow_sending_without_reply=None, width=None, height=None, protect_content=None, message_thread_id=None):
     method_url = r'sendAnimation'
     payload = {'chat_id': chat_id}
     files = None
@@ -721,12 +748,14 @@ async def send_animation(
         payload['height'] = height
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_voice(token, chat_id, voice, caption=None, duration=None, reply_to_message_id=None, reply_markup=None,
                parse_mode=None, disable_notification=None, timeout=None, caption_entities=None,
-               allow_sending_without_reply=None, protect_content=None):
+               allow_sending_without_reply=None, protect_content=None, message_thread_id=None):
     method_url = r'sendVoice'
     payload = {'chat_id': chat_id}
     files = None
@@ -754,11 +783,14 @@ async def send_voice(token, chat_id, voice, caption=None, duration=None, reply_t
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_video_note(token, chat_id, data, duration=None, length=None, reply_to_message_id=None, reply_markup=None,
-                    disable_notification=None, timeout=None, thumb=None, allow_sending_without_reply=None, protect_content=None):
+                    disable_notification=None, timeout=None, thumb=None, allow_sending_without_reply=None, protect_content=None,
+                    message_thread_id=None):
     method_url = r'sendVideoNote'
     payload = {'chat_id': chat_id}
     files = None
@@ -792,12 +824,14 @@ async def send_video_note(token, chat_id, data, duration=None, length=None, repl
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_audio(token, chat_id, audio, caption=None, duration=None, performer=None, title=None, reply_to_message_id=None,
                reply_markup=None, parse_mode=None, disable_notification=None, timeout=None, thumb=None,
-               caption_entities=None, allow_sending_without_reply=None, protect_content=None):
+               caption_entities=None, allow_sending_without_reply=None, protect_content=None, message_thread_id=None):
     method_url = r'sendAudio'
     payload = {'chat_id': chat_id}
     files = None
@@ -837,12 +871,15 @@ async def send_audio(token, chat_id, audio, caption=None, duration=None, perform
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_data(token, chat_id, data, data_type, reply_to_message_id=None, reply_markup=None, parse_mode=None,
               disable_notification=None, timeout=None, caption=None, thumb=None, caption_entities=None,
-              allow_sending_without_reply=None, disable_content_type_detection=None, visible_file_name=None, protect_content=None):
+              allow_sending_without_reply=None, disable_content_type_detection=None, visible_file_name=None, protect_content=None,
+              message_thread_id=None):
     method_url = await get_method_by_type(data_type)
     payload = {'chat_id': chat_id}
     files = None
@@ -881,6 +918,8 @@ async def send_data(token, chat_id, data, data_type, reply_to_message_id=None, r
         payload['protect_content'] = protect_content
     if method_url == 'sendDocument' and disable_content_type_detection is not None:
         payload['disable_content_type_detection'] = disable_content_type_detection
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
@@ -949,7 +988,7 @@ async def promote_chat_member(
         token, chat_id, user_id, can_change_info=None, can_post_messages=None,
         can_edit_messages=None, can_delete_messages=None, can_invite_users=None,
         can_restrict_members=None, can_pin_messages=None, can_promote_members=None,
-        is_anonymous=None, can_manage_chat=None, can_manage_video_chats=None):
+        is_anonymous=None, can_manage_chat=None, can_manage_video_chats=None, can_manage_topics=None):
     method_url = 'promoteChatMember'
     payload = {'chat_id': chat_id, 'user_id': user_id}
     if can_change_info is not None:
@@ -974,6 +1013,8 @@ async def promote_chat_member(
         payload['can_manage_chat'] = can_manage_chat
     if can_manage_video_chats is not None:
         payload['can_manage_video_chats'] = can_manage_video_chats
+    if can_manage_topics is not None:
+        payload['can_manage_topics'] = can_manage_topics
     return await _process_request(token, method_url, params=payload, method='post')
 
 
@@ -1285,7 +1326,7 @@ async def delete_message(token, chat_id, message_id, timeout=None):
 async def send_game(
         token, chat_id, game_short_name,
         disable_notification=None, reply_to_message_id=None, reply_markup=None, timeout=None,
-        allow_sending_without_reply=None, protect_content=None):
+        allow_sending_without_reply=None, protect_content=None, message_thread_id=None):
     method_url = r'sendGame'
     payload = {'chat_id': chat_id, 'game_short_name': game_short_name}
     if disable_notification is not None:
@@ -1300,6 +1341,8 @@ async def send_game(
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload)
 
 
@@ -1364,7 +1407,8 @@ async def send_invoice(
         need_name=None, need_phone_number=None, need_email=None, need_shipping_address=None,
         send_phone_number_to_provider = None, send_email_to_provider = None, is_flexible=None,
         disable_notification=None, reply_to_message_id=None, reply_markup=None, provider_data=None,
-        timeout=None, allow_sending_without_reply=None, max_tip_amount=None, suggested_tip_amounts=None, protect_content=None):
+        timeout=None, allow_sending_without_reply=None, max_tip_amount=None, suggested_tip_amounts=None, protect_content=None,
+        message_thread_id=None):
     """
     Use this method to send invoices. On success, the sent Message is returned.
     :param token: Bot's token (you don't need to fill this)
@@ -1445,6 +1489,8 @@ async def send_invoice(
         payload['suggested_tip_amounts'] = json.dumps(suggested_tip_amounts)
     if protect_content is not None:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload)
 
 
@@ -1653,7 +1699,7 @@ async def send_poll(
         is_anonymous = None, type = None, allows_multiple_answers = None, correct_option_id = None,
         explanation = None, explanation_parse_mode=None, open_period = None, close_date = None, is_closed = None,
         disable_notification=False, reply_to_message_id=None, allow_sending_without_reply=None,
-        reply_markup=None, timeout=None, explanation_entities=None, protect_content=None):
+        reply_markup=None, timeout=None, explanation_entities=None, protect_content=None, message_thread_id=None):
     method_url = r'sendPoll'
     payload = {
         'chat_id': str(chat_id),
@@ -1697,8 +1743,49 @@ async def send_poll(
             types.MessageEntity.to_list_of_dicts(explanation_entities))
     if protect_content:
         payload['protect_content'] = protect_content
+    if message_thread_id:
+        payload['message_thread_id'] = message_thread_id
     return await _process_request(token, method_url, params=payload)
 
+
+async def create_forum_topic(token, chat_id, name, icon_color=None, icon_custom_emoji_id=None):
+    method_url = r'createForumTopic'
+    payload = {'chat_id': chat_id, 'name': name}
+    if icon_color:
+        payload['icon_color'] = icon_color
+    if icon_custom_emoji_id:
+        payload['icon_custom_emoji_id'] = icon_custom_emoji_id
+    return await _process_request(token, method_url, params=payload)
+
+async def edit_forum_topic(token, chat_id, message_thread_id, name, icon_custom_emoji_id):
+    method_url = r'editForumTopic'
+    payload = {'chat_id': chat_id, 'message_thread_id': message_thread_id, 'name': name, 'icon_custom_emoji_id': icon_custom_emoji_id}
+    return await _process_request(token, method_url, params=payload)
+
+async def close_forum_topic(token, chat_id, message_thread_id):
+    method_url = r'closeForumTopic'
+    payload = {'chat_id': chat_id, 'message_thread_id': message_thread_id}
+    return await _process_request(token, method_url, params=payload)
+
+async def reopen_forum_topic(token, chat_id, message_thread_id):
+    method_url = r'reopenForumTopic'
+    payload = {'chat_id': chat_id, 'message_thread_id': message_thread_id}
+    return await _process_request(token, method_url, params=payload)
+
+async def delete_forum_topic(token, chat_id, message_thread_id):
+    method_url = r'deleteForumTopic'
+    payload = {'chat_id': chat_id, 'message_thread_id': message_thread_id}
+    return await _process_request(token, method_url, params=payload)
+
+async def unpin_all_forum_topic_messages(token, chat_id, message_thread_id):
+    method_url = r'unpinAllForumTopicMessages'
+    payload = {'chat_id': chat_id, 'message_thread_id': message_thread_id}
+    return await _process_request(token, method_url, params=payload)
+
+async def get_forum_topic_icon_stickers(token):
+    method_url = r'getForumTopicIconStickers'
+    return await _process_request(token, method_url)
+
 async def _convert_list_json_serializable(results):
     ret = ''
     for r in results:
